@@ -2,12 +2,13 @@
 import sys
 import os
 import optparse
+import re
 import unittest
 from xml.sax import saxutils
 
 from nose.pyversion import UNICODE_STRINGS
 from nose.tools import eq_
-from nose.plugins.xunit import Xunit, escape_cdata
+from nose.plugins.xunit import Xunit, escape_cdata, id_split
 from nose.exc import SkipTest
 from nose.config import Config
 
@@ -20,6 +21,8 @@ def mktest():
 
 mktest.__test__ = False
 
+time_taken = re.compile(r'\d\.\d\d')
+
 class TestEscaping(unittest.TestCase):
 
     def setUp(self):
@@ -41,8 +44,8 @@ class TestEscaping(unittest.TestCase):
         utf16 = self.x._quoteattr(u'Ivan Krsti\u0107')[1:-1]
 
         if UNICODE_STRINGS:
-	    # If all internal strings are unicode, then _quoteattr shouldn't
-	    # have changed anything.
+            # If all internal strings are unicode, then _quoteattr shouldn't
+            # have changed anything.
             eq_(utf16, u'Ivan Krsti\u0107')
         else:
             # to avoid big/little endian bytes, assert that we can put it back:
@@ -55,6 +58,26 @@ class TestEscaping(unittest.TestCase):
         eq_(self.x._quoteattr('foo\n\b\f\r'), '"foo%s??%s"' % (n, r))
         eq_(escape_cdata('foo\n\b\f\r'), 'foo\n??\r')
 
+class TestSplitId(unittest.TestCase):
+
+    def check_id_split(self, cls, name):
+        split = id_split('%s.%s' % (cls, name))
+        eq_(split[0], cls)
+        eq_(split[1], name)
+
+    def test_no_parenthesis(self):
+        self.check_id_split("test_parset", "test_args")
+
+    def test_no_dot_in_args(self):
+        self.check_id_split("test_parset", "test_args(('x', [1, 2]),)")
+
+    def test_dot_in_args(self):
+        self.check_id_split("test_parset", "test_args(('x.y', 1),)")
+
+    def test_grandchild_has_dot_in_args(self):
+        self.check_id_split("test_grandparset.test_parset",
+                            "test_args(('x.y', 1),)")
+
 class TestOptions(unittest.TestCase):
 
     def test_defaults(self):
@@ -107,13 +130,14 @@ class TestXMLOutputWithXML(unittest.TestCase):
         class DummyStream:
             pass
         self.x.report(DummyStream())
-        f = open(self.xmlfile, 'r')
-        return f.read()
+        f = open(self.xmlfile, 'rb')
+        data = f.read()
         f.close()
+        return data
 
     def test_addFailure(self):
         test = mktest()
-        self.x.startTest(test)
+        self.x.beforeTest(test)
         try:
             raise AssertionError("one is not 'equal' to two")
         except AssertionError:
@@ -135,7 +159,8 @@ class TestXMLOutputWithXML(unittest.TestCase):
             tc = tree.find("testcase")
             eq_(tc.attrib['classname'], "test_xunit.TC")
             eq_(tc.attrib['name'], "runTest")
-            assert int(tc.attrib['time']) >= 0
+            assert time_taken.match(tc.attrib['time']), (
+                        'Expected decimal time: %s' % tc.attrib['time'])
 
             err = tc.find("failure")
             eq_(err.attrib['type'], "%s.AssertionError" % (AssertionError.__module__,))
@@ -169,16 +194,17 @@ class TestXMLOutputWithXML(unittest.TestCase):
         if self.ET:
             tree = self.ET.fromstring(result)
             tc = tree.find("testcase")
-            eq_(tc.attrib['time'], "0")
+            assert time_taken.match(tc.attrib['time']), (
+                        'Expected decimal time: %s' % tc.attrib['time'])
         else:
             # this is a dumb test for 2.4-
             assert '<?xml version="1.0" encoding="UTF-8"?>' in result
             assert ('<testcase classname="test_xunit.TC" '
-                    'name="runTest" time="0">') in result
+                    'name="runTest" time="0') in result
 
     def test_addError(self):
         test = mktest()
-        self.x.startTest(test)
+        self.x.beforeTest(test)
         try:
             raise RuntimeError("some error happened")
         except RuntimeError:
@@ -200,7 +226,8 @@ class TestXMLOutputWithXML(unittest.TestCase):
             tc = tree.find("testcase")
             eq_(tc.attrib['classname'], "test_xunit.TC")
             eq_(tc.attrib['name'], "runTest")
-            assert int(tc.attrib['time']) >= 0
+            assert time_taken.match(tc.attrib['time']), (
+                        'Expected decimal time: %s' % tc.attrib['time'])
 
             err = tc.find("error")
             eq_(err.attrib['type'], "%s.RuntimeError" % (RuntimeError.__module__,))
@@ -217,6 +244,31 @@ class TestXMLOutputWithXML(unittest.TestCase):
             assert 'RuntimeError: some error happened' in result
             assert '</error></testcase></testsuite>' in result
 
+    def test_non_utf8_error(self):
+        # See http://code.google.com/p/python-nose/issues/detail?id=395
+        test = mktest()
+        self.x.beforeTest(test)
+        try:
+            raise RuntimeError(chr(128)) # cannot encode as utf8 
+        except RuntimeError:
+            some_err = sys.exc_info()
+        self.x.addError(test, some_err)
+        result = self.get_xml_report()
+        print repr(result)
+        if self.ET:
+            tree = self.ET.fromstring(result)
+            tc = tree.find("testcase")
+            err = tc.find("error")
+            if UNICODE_STRINGS:
+                eq_(err.attrib['message'],
+                    '\x80')
+            else:
+                eq_(err.attrib['message'],
+                    u'\ufffd')
+        else:
+            # this is a dumb test for 2.4-
+            assert 'RuntimeError: \xef\xbf\xbd' in result
+
     def test_addError_early(self):
         test = mktest()
         try:
@@ -224,6 +276,8 @@ class TestXMLOutputWithXML(unittest.TestCase):
         except RuntimeError:
             some_err = sys.exc_info()
 
+        self.x.startContext(None)
+
         # call addError without startTest
         # which can happen if setup() raises an error
         self.x.addError(test, some_err)
@@ -234,16 +288,17 @@ class TestXMLOutputWithXML(unittest.TestCase):
         if self.ET:
             tree = self.ET.fromstring(result)
             tc = tree.find("testcase")
-            eq_(tc.attrib['time'], "0")
+            assert time_taken.match(tc.attrib['time']), (
+                        'Expected decimal time: %s' % tc.attrib['time'])
         else:
             # this is a dumb test for 2.4-
             assert '<?xml version="1.0" encoding="UTF-8"?>' in result
             assert ('<testcase classname="test_xunit.TC" '
-                    'name="runTest" time="0">') in result
+                    'name="runTest" time="0') in result
 
     def test_addSuccess(self):
         test = mktest()
-        self.x.startTest(test)
+        self.x.beforeTest(test)
         self.x.addSuccess(test, (None,None,None))
 
         result = self.get_xml_report()
@@ -260,7 +315,8 @@ class TestXMLOutputWithXML(unittest.TestCase):
             tc = tree.find("testcase")
             eq_(tc.attrib['classname'], "test_xunit.TC")
             eq_(tc.attrib['name'], "runTest")
-            assert int(tc.attrib['time']) >= 0
+            assert time_taken.match(tc.attrib['time']), (
+                        'Expected decimal time: %s' % tc.attrib['time'])
         else:
             # this is a dumb test for 2.4-
             assert '<?xml version="1.0" encoding="UTF-8"?>' in result
@@ -280,10 +336,11 @@ class TestXMLOutputWithXML(unittest.TestCase):
         if self.ET:
             tree = self.ET.fromstring(result)
             tc = tree.find("testcase")
-            eq_(tc.attrib['time'], "0")
+            assert time_taken.match(tc.attrib['time']), (
+                        'Expected decimal time: %s' % tc.attrib['time'])
         else:
             # this is a dumb test for 2.4-
             assert '<?xml version="1.0" encoding="UTF-8"?>' in result
             assert ('<testcase classname="test_xunit.TC" '
-                    'name="runTest" time="0" />') in result
+                    'name="runTest" time="0') in result
 
